/*A MODEL FOR UTERINE SMOOTH MUSCLE CELLS
Copyright (c) 2011 Wing Chiu Tong <winnie.tong@newcastle.ac.uk>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

Developmental History:
MJ Taggart, A Blanks, S Kharche, AV Holden, B Wang, H Zhang (2007) BMC pregnancy Childbirth, 7(Suppl 1):S3
WC Tong, CY Choi, H Zhang, MJ Taggart (2008) Proc Physiol Soc, 11:PC145
WC Tong, CY Choi, H Zhang, MJ Taggart, AV Holden (2009) Reprod Sci, 16:285A
WC Tong, CY Choi, H Zhang, MJ Taggart, (2009) Reprod Sci, 16:284A-285A
WC Tong, CY Choi, S Karche, AV Holden, H Zhang, MJ Taggart (PLoS ONE, submitted, 2011)

Usage:	gcc -O3 -o USMC.exe USMC.c 

*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>

/* The definitions of all the dynamical variables, parameters and equations of the model are included in this h-file for clarity. */
#include "USMC.h"

/* Different stimulating protocols are included below. Each will reproduce the specific figures published in WC Tong, CY Choi, S Karche, AV Holden, H Zhang, MJ Taggart (PLoS ONE, submitted, 2011)
*/
/*#define FIGURE12*/
/*#define FIGURE13A*/
#define FIGURE13B
/*#define FIGURE13C*/

#define FILENAME "AP.dat"	/* Output filename */

#define dt	0.02		/* Time step (ms) */
#define	T_TOTAL	30000.0		/* Total integration time (ms) */	
#define T_NOUT	100		/* Print to file every T_NOUT time-steps */
#define T_PRINT	1000		/* Print to screen every T_PRINT (ms) */
	
#ifdef FIGURE12
#define COMMENT	"# Figure 12: AP at 30 C with a 2-sec current clamp \n"
#define ST_ON	1000.0		/* Time to turn on stimulus (ms) */
#define ST_OFF	3000.0		/* Time to turn off stimulus (ms) */
#define ST_BCL	0.0		/* Basic cycle length, require only for periodic stimulation (ms)*/
#define ST_BON	0.0		/* Time to turn on a stimulus during a periodic stimulation (ms) */
#define ST_BOFF	0.0		/* Time to turn off a stimulus during a periodic stimulation (ms) */
#define ST_HOLD	0.0		/* Holding level of the stimulus (pA/pF) */
#define ST_STEP	-0.5		/* Stepping level of the stimulus (pA/pF) */
#endif

#ifdef FIGURE13A
#define COMMENT	"# Figure 13a: AP at 30 C with a single current pulse \n"
#define	T_TOTAL	30000.0		
#define ST_ON	1000.0
#define ST_OFF	1020.0
#define ST_BCL	0.0
#define ST_BON	0.0
#define ST_BOFF	0.0
#define ST_HOLD	0.0
#define ST_STEP	-1.5
#endif

#ifdef FIGURE13B
#define COMMENT	"# Figure 13b: AP at 30 C with 4 periodic current pulses \n"
#define ST_ON	0.0
#define ST_OFF	10000.0
#define ST_BCL	2500.0
#define ST_BON	1000.0
#define ST_BOFF	1020.0
#define ST_HOLD	0.0
#define ST_STEP	-1.5
#endif

#ifdef FIGURE13C
#define COMMENT	"# Figure 13c: AP at 30 C with a 5-sec current clamp \n"
#define ST_ON	1000.0
#define ST_OFF	6000.0
#define ST_BCL	0.0
#define ST_BON	0.0
#define ST_BOFF	0.0
#define ST_HOLD	0.0
#define ST_STEP	-5
#endif

/* Variables for stimulation */
double	t_on	= ST_ON; 	
double	t_off	= ST_OFF; 	
double	bcl	= ST_BCL; 	
double	t_bon	= ST_BON; 	
double	t_boff	= ST_BOFF; 	
double  hold    = ST_HOLD;
double  step    = ST_STEP;

/* Functions for integration */
void initial_conditions() ;
void currents();		
void flux_Ca();		
void print_output(double time) ;
void ode_gates (double v,double cai);

/* Global variable for print to output file */
FILE *fit	= NULL; 

int main (int argc, char *argv[]) { 

	char name[200]	= FILENAME; 	
	double t	= 0.0;		/* Time (ms) */
	long int nout	= T_NOUT;

	int i=0;			
			
	/* Open a file for output, and print a comment of what the simulation is about both on screen and in the file. */
	fit = fopen(name, "w");
	printf("Output I(t) to %s\n",name);
	puts( COMMENT );
	fputs( COMMENT ,fit);

	/* Set initial condition */
	initial_conditions();

	/* output at t=0 */
	currents();			
	flux_Ca();			
	print_output (0.0);

	do {
	/* Apply stimulus */
	#ifdef FIGURE13B
	/*Periodic stimuli */
	sim=(st(t)*bst(t))?(step):(hold);
	#else
	/*Single stimulus */
	sim=((st(t))?(step):(hold));	
	#endif
		
		/* Integrate until time to print output or apply stimulus */
		do {
		currents();			
		flux_Ca();
		ode_gates(v,cai);			
		v += -dt*(sim+it);
		cai += -dt*jt;			
		i++;
		} while (i<nout);
	/* update time */	
	t+=dt*nout;
	i=0;	
	print_output(t);
	/* Print to screen periodically to signal how far the simulation is going.  */
	if (fmod(t,T_PRINT)<=0.0) printf("t=%g \n",t);

	} while (t<T_TOTAL);


	/* This gives the values at the final time step. */
	print_output(t);
	fclose(fit);
		
	puts("Done");
	return (0);
}

void initial_conditions() {
			
	v=-53.90915441282156;
	cai=0.0001161881607214449;
	m=0.1253518889572223;
	h=0.404599170710196;
	b=0.508117603077852;
	g=0.03582573962705717;
	d=0.01036961357784695;
	f1=0.9065941499695301;
	f2=0.9065967263076083;
	q=0.2060363247740295;
	r1=0.1922244113609531;
	r2=0.1932803618375963;
	p=0.1174074734567931;
	k1=0.9968385770271651;
	k2=0.9968408069904307;
	xa=0.0003569126518797985;
	xab=0.002220456569762898;
	s=0.0307583106982354;
	x=0.08785242843398365;
	y=0.002604864867063448;
	c=0.0003764413740731269;
	w=0.2345238135343783;

}

/* Compute currents */
void currents () {
	
	xina=ina(v);
	xical=ical(v,cai);
	xicat=icat(v);
	xik1=ik1(v);
	xik2=ik2(v);
	xBKa=BKa(v);
	xBKab=BKab(v);
	xika=ika(v);
	xih=ih(v);
	xicl=icl(v);
	xinscc=inscc(v);
	xinaca=0.5*inaca(v,cai); /* JNaCa = 2 INaCa for JCa*/
	xinak=inak(v);
	xib=ib(v);
	it=xina+xical+xicat+xik1+xik2+xBKa+xBKab+xika+xih+xicl+xinscc+xinaca+xinak+xib;
}

/* Compute Ca fluxes */
void flux_Ca() {
	jt=(((AV*Cm*buff)/(zca*frdy))*(xical+xicat+insca(v))-jnaca(v,cai)+jpmca(cai));
}

void print_output (double time) {

	/* Calculate force */
	/* Ft=Fmax*w; */
	/* Set force to zero at rest */
	Ft=Fmax*(w-0.2345);
		
	/* Calculate total IK */
	ik=xik1+xik2+xBKa+xBKab+xika+xib; 
	
	fprintf(fit,"%.10g %g %g %g %g %g %g ",time,v,cai,Ft,it,ik,sim);
	
	fputs("\n",fit);
	
}

/* Update dynamical variables. Euler Method is used below. */
void ode_gates (double v,double cai) {			
	m += dt*((mss(v)-m)/(mtc(v)));
	h += dt*((hss(v)-h)/(htc(v)));
	b += dt*((bss(v)-b)/(btc(v)));
	g += dt*((gss(v)-g)/(gtc(v)));
	d += dt*((dss(v)-d)/(dtc(v)));
	f1 += dt*((f1ss(v)-f1)/(f1tc(v)));
	f2 += dt*((f2ss(v)-f2)/(f2tc(v)));

	q += dt*((qss(v)-q)/(qtc(v)));
	r1 += dt*((r1ss(v)-r1)/(r1tc(v)));
	r2 += dt*((r2ss(v)-r2)/(r2tc(v)));
	p += dt*((pss(v)-p)/(ptc(v)));
	k1 += dt*((k1ss(v)-k1)/(k1tc(v)));
	k2 += dt*((k2ss(v)-k2)/(k2tc(v)));
	xa += dt*((xass(v,cai)-xa)/(xatc(v)));
	xab += dt*((xabss(v,cai)-xab)/(xabtc(v)));
	
	s += dt*((sss(v)-s)/(stc(v)));
	x += dt*((xss(v)-x)/(xtc(v)));
	y += dt*((yss(v)-y)/(ytc(v)));
	c += dt*((css(v,cai)-c)/(ctc(v)));
	w += dt* (wss(cai)-w)/wtc(cai);
}		

/* End of file*/
