/*A MODEL FOR UTERINE SMOOTH MUSCLE CELLS
Copyright (c) 2011 Wing Chiu Tong <winnie.tong@newcastle.ac.uk>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

Developmental History:
MJ Taggart, A Blanks, S Kharche, AV Holden, B Wang, H Zhang (2007) BMC pregnancy Childbirth, 7(Suppl 1):S3
WC Tong, CY Choi, H Zhang, MJ Taggart (2008) Proc Physiol Soc, 11:PC145
WC Tong, CY Choi, H Zhang, MJ Taggart, AV Holden (2009) Reprod Sci, 16:285A
WC Tong, CY Choi, H Zhang, MJ Taggart, (2009) Reprod Sci, 16:284A-285A
WC Tong, CY Choi, S Karche, AV Holden, H Zhang, MJ Taggart (PLoS ONE, submitted, 2011)

*/

double v	= 0;		/* Membrane potential (mV) */
double cai	= 0;		/* Intracellular Ca2+ (mM) */

/* --------------------------------------- */
/* Stimulus Functions */	
#define heav(x)	((x>=0.0)?1.0:0.0)
#define st(t)	(heav(t_off-t)*heav(t-t_on))
#define bst(t)	(heav(t_boff-fmod(t,bcl))*heav(fmod(t,bcl)-t_bon))

/* --------------------------------------- */
/* Constants */
#define zca	2.0 		/* Valency for Ca2+ */
#define zna	1.0 		/* Valency for Na+ */
#define zk	1.0		/* Valency for K+ */

#define R 	8314.0      	/* Universal Gas Constant (J/kmol*K) */
#define frdy	96485.0   	/* Faraday's Constant (C/mol) */
#define temp	308.0   	/* Temperature (K) */
#define Cm	1.0		/* Specific membrane capacitance (uF/cm2) */

/* ionic concentrations 
modified from Okabe et al 1999 Eur. J Pharmacology 376:101-108*/
double ki	= 140.0;	/* Intracellular K+ (mM) */
double nai	= 4.0;		/* Intracellular Na+ (mM) */
double cli	= 46.0;		/* Intracellular Cl- (mM) */
double ko	= 6.0;		/* Extracellular K+ (mM) */
double cao	= 2.5;		/* Extracellular Ca2+ (mM) */
double nao	= 130.0;	/* Extracellular Na+ (mM) */
double clo	= 130.0;	/* Extracellular Cl- (mM) */
double mgo	= 0.5;		/* Extracellular Mg2+ (mM) */

double buff	= 0.015;	/* Proportion of free Calcium ions from calcium flux */
double AV	= 4.0;		/* Area to Volume ratio (cm^-1) */

/* --------------------------------------- 
  Note: Functions with 'ss' suffix are steady-state functions: e.g wss(cai), and these are dimensionless.
  Functions with 'tc' suffix are time constant functions: e.g wtc(cai), and these are in ms.
   --------------------------------------- */

/* --------------------------------------- */
/* Calcium-dependent force */
double w	= 0;		/* dynamical variable for force */
double Fmax	= 3.0;		/* maxiumal force (uN) */
double FKm	= 161.301;	/* half saturation calcium conc. (nM) */
double Fn	= 3.60205;	/* Hill coeff  (dimensionless) */
#define wss(cai)	(1.0/(1.0+pow(((FKm*1e-6)/cai),Fn)))					/* steady-state function for w gate */
#define wtc(cai)	(4000.0*(0.234845+((1.0-0.234845)/(1.0+pow((cai/(FKm*1e-6)),Fn)))))	/* time constant function for w gate */


/* --------------------------------------- */
/* Sodium current - INa */
double m	= 0;		/* dynamical variable for INa activation */
double h	= 0;		/* dynamical variable for INa inactivation */
double gna	= 0; 		/* 0-0.12; maximal conductance for INa (nS/pF)  */
#define mss(v)	(1.0/(1.0+exp(-(v+35.9584)/9.24013)))		/* steady-state function for m gate */
#define hss(v)	(1.0/(1.0+exp((v+57.0)/8.0)))			/* steady-state function for h gate*/
#define mtc(v)	(0.25+7.0/(1.0+exp((v+38.0)/10.0)))		/* time constant function for m gate */
#define htc(v)	(0.9+1002.85/(1.0+((v+47.5)/1.5)*((v+47.5)/1.5))) /* time constant function for h gate */
#define ena	((R*temp/frdy)*log(nao/nai))	/* reversal potential for sodium (mV) */
#define ina(v)	(gna*(m*m*m)*h*(v-ena))		/* INa */

/* --------------------------------------- */
/* L-type calcium current - ICaL */
double d	= 0;		/* dynamical variable for ICaL activation */
double f1	= 0;		/* dynamical variable for ICaL fast inactivation */
double f2	= 0;		/* dynamical variable for ICaL slow inactivation */
double gcal	= 0.6; 		/* Maximal conductance for ICaL (nS/pF)  */
double ecal	= 45.0;		/* reversal potential for ICaL (mV) */
double kmca	= 0.001;	/*  half saturation concentration for ICaL Ca inhibition (mM) */
#define fca(cai) (1.0/(1.0+pow((cai/kmca),4.0)))	/* Ca inhibition in ICaL */
#define dss(v)	(1.0/(1.0+exp(-(v+22.0)/7.0)))		/* steady-state for d*/
#define f1ss(v)	(1.0/(1.0+exp((v+38.0)/7.0)))		/* steady-state for f1*/
#define f2ss(v)	(f1ss(v))				/* steady-state for f2*/
#define dtc(v)	(2.29+5.7/(1.0+((v+29.97)/9.0)*((v+29.97)/9.0)))	/* time constant for d */
#define f1tc(v)	(12.0)							/* time constant for f1 */
#define f2tc(v)	(90.9699*(1.0-(1.0/(1.0+exp((v+13.9629)/45.3782)))*(1.0/(1.0+exp(-(v+9.49866)/3.3945)))))	/* time constant for f2 */
#define ical(v,cai)	(gcal*fca(cai)*(d*d)*(0.8*f1+0.2*f2)*(v-ecal))	/* ICaL  */


/* --------------------------------------- */
/* T-type calcium current - ICaT */
double b	= 0;		/* dynamical variable for ICaT activation  */
double g	= 0;		/* dynamical variable for ICaT inactivation */
double gcat	= 0.058;	/* Maximal conductance for ICaT (nS/pF)  */
double ecat	= 42.0;		/* reversal potential for ICaT (mV) */
#define bss(v)	(1.0/(1.0+exp(-(v+54.23)/9.88)))			/* steady-state for b*/
#define gss(v)	(0.02+(1.0-0.02)/(1.0+exp((v+72.978)/4.64)))		/* steady-state for g*/
#define btc(v)	(0.45+3.9/(1.0+((v+66.0)/26.0)*((v+66.0)/26.0)))	/* time constant for b */
#define gtc(v)	(150.0*(1.0-(1.0/(1.0+exp((v-417.43)/203.18)))*(1.0/(1.0+exp(-(v+61.11)/8.07)))))	/* time constant for g*/
#define icat(v)	(gcat*(b*b)*g*(v-ecat))  /* ICaT */

/* --------------------------------------- */
/* Potassium conductances */
double gk	= 0.8;		/* Maximal conductance for total voltage-gated K currents (IK1, IK2, IKa) (nS/pF)  */
double gkca	= 0.8;		/* Maximal conductance for total calcium-dependent K currents (BKa, BKab) (nS/pF)  */

/* --------------------------------------- */
/* background current - Ib */
double gb	= 0.004; 			/* background current conductances (nS/pF) */
#define ek	((R*temp/frdy)*log(ko/ki)) 	/* reversal potentials for potassium (mV) */
#define ib(v)	(gb*(v-ek))			/* Ib */

/* --------------------------------------- */
/* Voltage-gated potassiun - IK1 */
double q	= 0;		/* dynamical variable for IK1 activation  */
double r1	= 0;		/* dynamical variable for IK1 fast inactivation  */
double r2	= 0;		/* dynamical variable for IK1 slow inactivation  */
double gk1	= 0.65;		/* Conductance ratios between ik1, ik2, ika (dimentionless) */
#define qss(v)	(0.978613/(1.0+exp(-(v+18.6736)/26.6606)))	/* steady-state for q*/
#define r1ss(v)	(1.0/(1.0+exp((v+63.0)/6.3)))			/* steady-state for r1*/
#define r2ss(v)	(r1ss(v))					/* steady-state for r2*/
#define qtc(v)	(500.0/(1.0+((v+60.71)/15.79)*((v+60.71)/15.79)))		/* time constant for q */
#define r1tc(v)	(5000.0/(1.0+((v+62.7133)/35.8611)*((v+62.7133)/35.8611)))	/* time constant for r1 */
#define r2tc(v)	(30000.0+220000.0/(1.0+exp((v+22.0)/4.0)))			/* time constant for r2 */
#define ik1(v)	(gk*gk1*(q*q)*(0.38*r1+0.63*r2)*(v-ek))  /* IK1 */


/* --------------------------------------- */
/* Voltage-gated potassium - IK2 */
double p	= 0;		/* dynamical variable for IK2 activation */
double k1	= 0;		/* dynamical variable for IK2 fast inactivation  */
double k2	= 0;		/* dynamical variable for IK2 slow inactivation */
double gk2	= 0.04;		/* Conductance ratios between ik1, ik2, ika (dimensionless) */
#define pss(v)	(0.948/(1.0+exp(-(v+17.91)/18.4)))	/* steady-state for p*/
#define k1ss(v)	(1.0/(1.0+exp((v+21.2)/5.7)))		/* steady-state for k1*/
#define k2ss(v)	(k1ss(v))				/* steady-state for k2*/
#define ptc(v)	(100.0/(1.0+((v+64.1)/28.67)*((v+64.1)/28.67)))								/* time constant for p */
#define k1tc(v)	(1.0e6*(1.0-(1.0/(1.0+exp((v-315.0)/50.0)))*(1.0/(1.0+exp(-(v+74.9)/8.0)))))				/* time constant for k1 */
#define k2tc(v)	(1000.0*2500.0*(1.0-(1.0/(1.0+exp((v-132.868)/25.3992)))*(1.0/(1.0+exp(-(v+24.9203)/2.67915)))))	/* time constant for k2 */
#define ik2(v)	(gk*gk2*(p*p)*(0.75*k1+0.25*k2)*(v-ek)) /* IK2 */


/* --------------------------------------- */
/* BK current, consists of subunit alpha - BKa  */
double xa	= 0;		/* dynamical variable for BKa */
double gbka	= 0.2;		/* Conductance ratios between BKa, BKab (dimensionless) */
#define xass_z(cai)	(-0.749234/(1.0+((cai*1000.0-0.0630535)/0.161942)*((cai*1000.0-0.0630535)/0.161942))+8.38384/(1.0+((cai*1000.0+1538.29)/739.057)*((cai*1000.0+1538.29)/739.057)))	/* gating charge for xa*/
#define xass_vh(cai)	(5011.47/(1.0+pow(((cai*1000.0+0.237503)/0.000239278),0.42291))-37.5137)	/* half-activation constant for xa*/
#define xass(v,cai)	(1.0/(1.0+exp(-xass_z(cai)*frdy*(v-xass_vh(cai))/(R*temp))))			/* steady-state for xa*/
#define xatc(v)		(2.40914/(1.0+((v-158.779)/(-52.1497))*((v-158.779)/(-52.1497))))		/* time constant for xa */
#define BKa(v)		(gkca*gbka*xa*(v-ek)) /* BKa */

/* --------------------------------------- */
/* BK current, consists of subunits alpha and beta1  - BKab  */
double xab	= 0;		/* dynamical variable for BKab */
double gbkab	= 0.1;		/* Conductance ratios between BKa, BKab (dimensionless) */
#define xabss_z(cai)	(-0.681249/(1.0+((cai*1000.0-0.218988)/0.428335)*((cai*1000.0-0.218988)/0.428335))+1.40001/(1.0+((cai*1000.0+228.71)/684.946)*((cai*1000.0+228.71)/684.946))) /* gating charge for xab*/
#define xabss_vh(cai)	(8540.23/(1.0+pow(((cai*1000.0+0.401189)/0.00399115),0.668054))-109.275)	/* half-activation constant for xab*/
#define xabss(v,cai)	(1.0/(1.0+exp(-xabss_z(cai)*frdy*(v-xabss_vh(cai))/(R*temp))))		/* steady-state for xab */
#define xabtc(v)	(13.8049/(1.0+((v-153.019)/66.4952)*((v-153.019)/66.4952)))		/* time constant for xab */
#define BKab(v)		(gkca*gbkab*xab*(v-ek)) /* BKab */

/* --------------------------------------- */
/* A-type transient potassium current - IKa */
double s	= 0;		/* dynamical variable for IKa activation */
double x	= 0;		/* dynamical variable for IKa inactivation  */
double gka	= 0.2;		/* Conductance ratios between ik1, ik2, ika (dimensionless) */
#define sss(v)	(1.0/(1.0+exp(-(v+27.79)/7.57)))	/* steady-state for s*/
#define xss(v)	(0.02+0.98/(1.0+exp((v+69.5)/6.0)))	/* steady-state for x*/
#define stc(v)	(17.0/(1.0+((v+20.5232)/35.0)*((v+20.5232)/35.0))) 		/* time constant for s */
#define xtc(v)	(7.5+10.0/(1.0+((v+34.1765)/120.0)*((v+34.1765)/120.0)))	/* time constant for x */
#define ika(v)	(gk*gka*s*x*(v-ek))  /* IKa */


/* --------------------------------------- */
/* Hyperpolarization-activated current - Ih */
double y	= 0;		/* dynamical variable for Ih activation */
double gh	= 0.0542;	/* Maximal conductance for Ih (nS/pF)  */
#define PK	1.0		/* Permeability ratios for K+ */
#define PNa	0.35		/* Permeability ratios for Na+ */
#define yss(v)	(1.0/(1.0+exp((v+105.39)/8.6553)))	/* steady-state for y*/
#define ya(v)	(3.5e-6*exp(-0.0497*v))
#define yb(v)	(0.04003*exp(0.05211*v))
#define ytc(v)	(1.0/(ya(v)+yb(v)))			/* time constant for y */
#define eh	((R*temp/frdy)*log((ko+(PNa/PK)*nao)/(ki+(PNa/PK)*nai)))	/* reversal potentials (mV) */
#define ih(v)	(gh*y*(v-eh))  /* Ih */


/* --------------------------------------- */
/* common funcion for ICl, INSCC INaK, INaCa */
#define vFRT(v)		((v*frdy)/(R*temp))

/* --------------------------------------- */
/* Calcium-dependent Chloride current - ICl */
double c	= 0;		/* dynamical variable for ICl activation */
double gcl	= 0.1875;	/* Maximal conductance for ICl (nS/pF)  */
#define K1cl(v)		(0.0006*exp(2.53*vFRT(v)))
#define K2cl(v)		(0.1*exp(-5.0*vFRT(v)))
#define css(v,cai)	(1.0/(1.0+K2cl(v)*((K1cl(v)/cai)*(K1cl(v)/cai)+K1cl(v)/cai+1.0)))		/* steady-state for c*/
#define ctc(v)		(-160.0+210.0/(1.0+exp((v+4.56)/11.62))+170.0/(1.0+exp(-(v+25.5)/11.62)))	/* time constant for c */
#define ecl		(((R*temp)/frdy)*log(cli/clo))	/* reversal potentials (mV) */
#define icl(v)		(gcl*c*(v-ecl)) /* ICL */

/* --------------------------------------- */
/* Non-specific cation current	- INSCC */
double gns	= 0.0123;	/* Maximal conductance for INSCC (nS/pF)  */
double gl	= 0.0;		/* Maximal conductance for the leak component of INSCC (nS/pF)  */
#define PnsK	1.3		/* Permeability of K+ */
#define PnsNa	0.9		/* Permeability of Na+ */
#define PnsCa	0.89		/* Permeability of Ca2+ */
#define PnsCs	1.0		/* Permeability of Cs+ */
#define gnsCa	0.5		/* Conductance ratio of Ca2+ in INSCC */
#define gnsNa	1.0		/* Conductance ratio of Na+ in INSCC */
#define gnsK	1.19		/* Conductance ratio of K+ in INSCC */
#define gnsCs	1.6		/* Conductance ratio of Cs+ in INSCC */
#define fmg(mgo)	(0.108043+0.903902/(1.0+pow((mgo/0.281007),1.29834))) /* Magnesium inhibition of INSCC */
#define enscc(v)	((R*temp/frdy)*log((PnsK*ko+PnsNa*nao+4*PnsCa*(1.0/(1.0+exp(vFRT(v))))*cao)/(PnsK*ki+PnsNa*nai+4*PnsCa*(1.0/(1.0+exp(vFRT(v))))*cai)))	/* reversal potentials (mV) */
#define gs_x(x)		((0.03/(1+(150.0/(x+1e-8))*(150.0/(x+1e-8))))/0.0123)	/* Extracellular ionic concentrations dependency on INSCC conductance */
#define gs_ca(x)	((0.03/(1+(150.0/(x+1e-8))*(150.0/(x+1e-8))))/0.000525) /* Extracellular calcium concentrations dependency on INSCC conductance */
#define insca(v)	(fmg(mgo)*(gs_ca(cao)*gnsCa)*gns*(v-(enscc(v))))	/* Ca component of INSCC */
#define insna(v)	(fmg(mgo)*(gs_x(nao)*gnsNa)*gns*(v-(enscc(v))))		/* Na component of INSCC */
#define insk(v)		(fmg(mgo)*(gs_x(ko)*gnsK)*gns*(v-(enscc(v))))		/* K component of INSCC */
#define il(v)		(fmg(mgo)*(gl)*(v-(enscc(v))))				/* leak component of INSCC */
#define inscc(v)	(insca(v)+insna(v)+insk(v)+il(v))			/* INSCC */

/* --------------------------------------- */
/* Na-K pump */
double ginak	= 1.7;		/* Maximal current for INaK (pA/pF)  */
#define nakKmko		2.0	/* Half-saturation concentration for ko dependency */
#define nakKmnai	22.0	/* Half-saturation concentration for Nai dependency */
#define fnak(v)		(1.0/(1.0+0.1245*exp(-0.1*vFRT(v))+2.19e-3*(exp(nao/49.71))*exp(-1.9*vFRT(v)))) /* Voltage-dependency of INaK */
#define knak		(1.0/(1.0+pow((nakKmko/ko),1.5))) /* ko-dependency of INaK */
#define nnak		(1.0/(1.0+pow((nakKmnai/nai),2))) /* nai-dependency of INaK */
#define inak(v)		(ginak*knak*nnak*fnak(v))	/* INaK */

/* --------------------------------------- */
/* Na-Ca exchanger */
double Jnaca	= 3.5e-6;	/* maximal calium flux via Na-Ca exchanger  (mM/ms) */
double Kmallo	= 0.0003;	/* half saturation conc. for calcium allosteric factor  (mM) */
double nallo	= 4;		/* Hill coeff. of allosteric regulation in   (dimensionless) */
double ksat	= 0.27;		/* saturation factor at very negative potential */
double xgamma	= 0.35;		/* partition parameter  */
double Kmnai	= 30.0;		/* dissocation constants for intracellular Na, Ca ions (mM) */
double Kmcai	= 0.007;	/* dissocation constants for intracellular Na, Ca ions (mM) */
double Kmnao	= 87.5;		/* dissocation constants for extracellular Na, Ca ions (mM) */
double Kmcao	= 1.3;		/* dissocation constants for extracellular Na, Ca ions (mM) */
#define f1naca(v)	(exp((xgamma-1.0)*vFRT(v)))
#define f2naca(v)	(exp(xgamma*vFRT(v)))
#define fallo(cai)	(1.0/(1.0+pow((Kmallo/cai),nallo)))
#define naca_Eup(v)	(Jnaca*((nai*nai*nai)*cao*f2naca(v)-(nao*nao*nao)*cai*f1naca(v)))
#define naca_Ed1(v)	(1.0+ksat*f1naca(v))
#define naca_Ed2(cai)	(Kmcao*(nai*nai*nai)+(Kmnao*Kmnao*Kmnao)*cai+(Kmnai*Kmnai*Kmnai)*cao*(1.0+cai/Kmcai))
#define naca_Ed3(cai)	(cao*(nai*nai*nai)+(nao*nao*nao)*cai+(nao*nao*nao)*Kmcai*(1.0+(nai/Kmnai)*(nai/Kmnai)*(nai/Kmnai)))
#define jnaca(v,cai)	(fallo(cai)*naca_Eup(v)/(naca_Ed1(v)*(naca_Ed2(cai)+naca_Ed3(cai))))	/* Calcium flux via Na-Ca exchanger (mM/ms)*/
#define inaca(v,cai)	(jnaca(v,cai)*((zca*frdy)/(AV*Cm*buff))) 				/* Current via Na-Ca exchanger (pA/pF) */

/* --------------------------------------- */
/* Plasma Membrane Calcium ATPase - PMCA */
double Jpmca	= 3.5e-7;	/* Maximal calium flux  via PMCA (mM/ms)*/
double Kmpmca	= 0.0005;	/* half saturation conc. (mM)*/
double npmca	= 2;		/* Hill Coeff */
#define jpmca(cai)	(Jpmca/(1.0+pow((Kmpmca/cai),npmca)))  /* Calcium flux via PMCA (mM/ms) */


/* --------------------------------------- */
/* global variables for calculations */
double xina	= 0;
double xical	= 0;
double xicat	= 0;
double xik1	= 0;
double xik2	= 0;
double xBKa	= 0;
double xBKab	= 0;
double xika	= 0;
double xih	= 0;
double xicl	= 0;
double xinscc	= 0;
double xinaca	= 0;
double xinak	= 0;
double xib	= 0;
double ik	= 0;
double it	= 0;
double jt	= 0;
double Ft	= 0;
double sim	= 0;

/* End of file*/
